﻿using C1.WPF.DataGrid;
using System;
using System.Collections.Generic;

namespace WpfDataGridGroupedSummary
{
    public interface IComputeGroup
    {
        object Compute(DataGridRowCollection rows, DataGridBoundColumn column, bool recursive);
    }

    public class ComputeSUM : IComputeGroup
    {

        #region IComputeGroup Members

        public object Compute(DataGridRowCollection rows, DataGridBoundColumn column, bool recursive)
        {
            double sum = 0;
            foreach (var row in rows.AsEnumerable(row => row.Type == DataGridRowType.Item, groupRow => recursive))
            {
                try
                {
                    sum += (double)row.DataItem.GetType().GetProperty(column.Binding.Path.Path).GetValue(row.DataItem, null);
                }
                catch (NullReferenceException)
                { 
                    // too much assumptions here
                }
            }
            return sum;
        }

        #endregion
    }

    public class ComputeCOUNT : IComputeGroup
    {

        #region IComputeGroup Members

        public object Compute(DataGridRowCollection rows, DataGridBoundColumn column, bool recursive)
        {
            int count = 0;
            foreach (var row in rows.AsEnumerable(row => row.Type == DataGridRowType.Item, groupRow => recursive))
            {
                count += 1;
            }
            return count;
        }

        #endregion
    }

    public class ComputeAVG : IComputeGroup
    {

        #region IComputeGroup Members

        public object Compute(DataGridRowCollection rows, DataGridBoundColumn column, bool recursive)
        {
            double sum = 0, count = 0;
            foreach (var row in rows.AsEnumerable(row => row.Type == DataGridRowType.Item, groupRow => recursive))
            {
                try
                {
                    count += 1;
                    sum += (double)row.DataItem.GetType().GetProperty(column.Binding.Path.Path).GetValue(row.DataItem, null);
                }
                catch (NullReferenceException) 
                {
                    // too much assumptions here
                }
            }
            return sum / count;
        }

        #endregion
    }

    public class ComputeMIN : IComputeGroup
    {

        #region IComputeGroup Members

        public object Compute(DataGridRowCollection rows, DataGridBoundColumn column, bool recursive)
        {
            object min = null;
            foreach (var row in rows.AsEnumerable(row => row.Type == DataGridRowType.Item, groupRow => recursive))
            {
                try
                {
                    var value = row.DataItem.GetType().GetProperty(column.Binding.Path.Path).GetValue(row.DataItem, null);
                    if (value is IComparable)
                    {
                        if (min == null || ((IComparable)value).CompareTo(min) < 0)
                        {
                            min = value;
                        }
                    }
                }
                catch (NullReferenceException) 
                {
                    // too much assumptions here
                }
            }
            return min;
        }

        #endregion
    }

    public class ComputeMAX : IComputeGroup
    {

        #region IComputeGroup Members

        public object Compute(DataGridRowCollection rows, DataGridBoundColumn column, bool recursive)
        {
            object max = null;
            foreach (var row in rows.AsEnumerable(row => row.Type == DataGridRowType.Item, groupRow => recursive))
            {
                try
                {
                    var value = row.DataItem.GetType().GetProperty(column.Binding.Path.Path).GetValue(row.DataItem, null);
                    if (value is IComparable)
                    {
                        if (max == null || ((IComparable)value).CompareTo(max) > 0)
                        {
                            max = value;
                        }
                    }
                }
                catch (NullReferenceException)
                {
                    // too much assumptions here
                }
            }
            return max;
        }

        #endregion
    }

    public class ComputeDISCTINCT : IComputeGroup
    {

        #region IComputeGroup Members

        public object Compute(DataGridRowCollection rows, DataGridBoundColumn column, bool recursive)
        {
            List<object> disctincts = new List<object>();
            foreach (var row in rows.AsEnumerable(row => row.Type == DataGridRowType.Item, groupRow => recursive))
            {
                try
                {
                    var value = row.DataItem.GetType().GetProperty(column.Binding.Path.Path).GetValue(row.DataItem, null);
                    if (value is IComparable)
                    {
                        bool found = false;
                        foreach (var disctinct in disctincts)
                        {
                            if (((IComparable)value).CompareTo(disctinct) == 0)
                            {
                                found = true;
                                break;
                            }
                        }
                        if (!found)
                        {
                            disctincts.Add(value);
                        }
                    }
                }
                catch (NullReferenceException)
                {
                    // too much assumptions here
                }
            }
            return disctincts.Count;
        }

        #endregion
    }
}
