﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;
using System.Xml;
using System.IO;
using C1.WPF.DataGrid;

namespace WpfDataGridGroupedSummary
{
    public class DataGridGroupWithSummaryRow : DataGridGroupRow
    {
        public DataGridGroupWithSummaryRow()
        {
            Summaries = new Dictionary<DataGridColumn, IComputeGroup>();
        }

        protected override void OnLoaded()
        {
            base.OnLoaded();
            CellStyle = cellStyle;
        }

        public Dictionary<DataGridColumn, IComputeGroup> Summaries { get; private set; }

        protected override object GetRowPresenterRecyclingKey()
        {
            return typeof(DataGridGroupWithSummaryRow);
        }

        protected override bool HasCellPresenter(DataGridColumn column)
        {
            if (Summaries.ContainsKey(column) && column is DataGridBoundColumn)
            {
                return true;
            }
            return false;
        }

        protected override object GetCellContentRecyclingKey(DataGridColumn column)
        {
            return typeof(TextBlock);
        }

        protected override FrameworkElement CreateCellContent(DataGridColumn column)
        {
            return new TextBlock();
        }

        protected override void BindCellContent(FrameworkElement cellContent, DataGridColumn column)
        {
            TextBlock txt = (TextBlock)cellContent;
            txt.HorizontalAlignment = column.HorizontalAlignment;
            txt.VerticalAlignment = column.VerticalAlignment;
            var value = Summaries[column].Compute(this.Rows, column as DataGridBoundColumn, false);
            txt.Text = value is double ? ((double)value).ToString((column as DataGridBoundColumn).Format) : value.ToString();
        }

        private static Style cellStyle = null;

        static DataGridGroupWithSummaryRow()
        {
            cellStyle = (Style)XamlReader.Load(XmlReader.Create(new StringReader(@"<Style xmlns=""http://schemas.microsoft.com/winfx/2006/xaml/presentation""
                                 xmlns:x=""http://schemas.microsoft.com/winfx/2006/xaml"" 
                                 xmlns:vsm=""" + "clr-namespace:System.Windows;assembly=WPFToolkit" + @""" " +
                                 @"xmlns:c1DataGrid=""clr-namespace:C1.WPF.DataGrid;assembly=C1.WPF.DataGrid""
                                 TargetType=""c1DataGrid:DataGridCellPresenter"">
                                <Setter Property=""IsHitTestVisible"" Value=""False""/>
                                <Setter Property=""Template"">
                                    <Setter.Value>
                                        <ControlTemplate TargetType=""c1DataGrid:DataGridCellPresenter"">
                                            <Grid>
                                                <vsm:VisualStateManager.VisualStateGroups>
                                                    <vsm:VisualStateGroup x:Name=""SelectionStates"">
                                                        <vsm:VisualState x:Name=""Editing"">
                                                        </vsm:VisualState>
                                                        <vsm:VisualState x:Name=""Current"">
                                                        </vsm:VisualState>
                                                        <vsm:VisualState x:Name=""Normal"">
                                                        </vsm:VisualState>
                                                    </vsm:VisualStateGroup>
                                                </vsm:VisualStateManager.VisualStateGroups>
                                                <Grid.ColumnDefinitions>
                                                    <ColumnDefinition Width=""*"" />
                                                    <ColumnDefinition Width=""Auto"" />
                                                </Grid.ColumnDefinitions>
                                                <Grid Margin=""{TemplateBinding Padding}"">
                                                    <Border Background=""Black"" Opacity=""0"" />
                                                    <Border Background=""{TemplateBinding Background}"" BorderBrush=""{TemplateBinding BorderBrush}"" 
							                      BorderThickness=""{TemplateBinding BorderThickness}"" CornerRadius=""{TemplateBinding CornerRadius}""/>
                                                    <ContentPresenter 
                                                    Margin=""{TemplateBinding BorderThickness}""
                                                    HorizontalAlignment=""{TemplateBinding HorizontalContentAlignment}"" 
                                                    VerticalAlignment=""{TemplateBinding VerticalContentAlignment}"" 
                                                    Content=""{TemplateBinding Content}"" />
                                                </Grid>
                                            </Grid>
                                        </ControlTemplate>
                                    </Setter.Value>
                                </Setter>
                            </Style>")));
        }
    }
}
