﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;
using System.Xml;
using System.IO;
using C1.WPF.DataGrid;

namespace WpfDataGridGroupedSummary
{
    public class DataGridSummaryRow 
        : DataGridRow
    {
        public Dictionary<DataGridColumn, IComputeGroup> Summaries { get; private set; }

        public bool UpdateWhenFilters { get; set; }

        #region ** initialization

        public DataGridSummaryRow()
        {
            Summaries = new Dictionary<DataGridColumn, IComputeGroup>();
        }

        protected override void OnLoaded()
        {
            base.OnLoaded();

            IsEditable = false;
            Height = DataGrid.GroupRowHeight;
            ShowVerticalFreezingSeparator = false;
            base.CellStyle = DataGridSummaryRow.CellStyle;
            base.RowStyle = DataGridSummaryRow.RowStyle;

            // it leaks memory, we need to remove this event somewhere
            this.ParentCollection.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(ParentCollection_CollectionChanged);
            //DataGrid.FilterChanged += new System.EventHandler<DataGridFilterChangedEventArgs>(DataGrid_FilterChanged);
        }

        void ParentCollection_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {            
        //}
        //void DataGrid_FilterChanged(object sender, DataGridFilterChangedEventArgs e)
        //{
            if (UpdateWhenFilters)
            {                
                foreach (var col in this.Summaries.Keys)
                {
                    var cellPresenter = GetCellPresenter(col);
                    var txt = cellPresenter.Content as TextBlock;
                    if (txt != null)
                    {
                        SetComputedValue(col, txt);
                    }
                }
            }
        }

        #endregion

        #region ** Row Presenter

        protected override object GetRowPresenterRecyclingKey()
        {
            return typeof(DataGridSummaryRow);
        }

        protected override DataGridRowPresenter CreateRowPresenter()
        {
            return new DataGridSelectableRowPresenter();
        }

        #endregion

        #region ** Cell Content Presenter

        protected override bool HasCellPresenter(DataGridColumn column)
        {
            if (Summaries.ContainsKey(column) && column is DataGridBoundColumn)
            {
                return true;
            }
            return false;
        }

        protected override object GetCellContentRecyclingKey(DataGridColumn column)
        {
            return typeof(TextBlock);
        }

        protected override FrameworkElement CreateCellContent(DataGridColumn column)
        {
            return new TextBlock();
        }

        protected override void BindCellContent(FrameworkElement cellContent, DataGridColumn column)
        {
            TextBlock txt = (TextBlock)cellContent;
            txt.HorizontalAlignment = column.HorizontalAlignment;
            txt.VerticalAlignment = column.VerticalAlignment;
            SetComputedValue(column, txt);
        }

        #endregion

        #region ** compute values

        private void SetComputedValue(DataGridColumn column, TextBlock txt)
        {
            var value = Summaries[column].Compute(this.ParentCollection, column as DataGridBoundColumn, false);
            txt.Text = value is double && column is DataGridNumericColumn
                     ? ((double)value).ToString((column as DataGridNumericColumn).Format)
                     : value.ToString();
        }

        #endregion

        #region ** static properties

        private static Style CellStyle = null;
        private static Style RowStyle = null;

        static DataGridSummaryRow()
        {
            CellStyle = (Style)XamlReader.Load(XmlReader.Create(new StringReader(@"<Style xmlns=""http://schemas.microsoft.com/winfx/2006/xaml/presentation""
                                 xmlns:x=""http://schemas.microsoft.com/winfx/2006/xaml"" 
                                 xmlns:vsm=""" + "clr-namespace:System.Windows;assembly=WPFToolkit" + @""" " +
                                 @"xmlns:c1DataGrid=""clr-namespace:C1.WPF.DataGrid;assembly=C1.WPF.DataGrid""
                                 TargetType=""c1DataGrid:DataGridCellPresenter"">
                                <Setter Property=""IsHitTestVisible"" Value=""False""/>
                                <Setter Property=""Template"">
                                    <Setter.Value>
                                        <ControlTemplate TargetType=""c1DataGrid:DataGridCellPresenter"">
                                            <Grid>
                                                <vsm:VisualStateManager.VisualStateGroups>
                                                    <vsm:VisualStateGroup x:Name=""SelectionStates"">
                                                        <vsm:VisualState x:Name=""Editing"">
                                                        </vsm:VisualState>
                                                        <vsm:VisualState x:Name=""Current"">
                                                        </vsm:VisualState>
                                                        <vsm:VisualState x:Name=""Normal"">
                                                        </vsm:VisualState>
                                                    </vsm:VisualStateGroup>
                                                </vsm:VisualStateManager.VisualStateGroups>
                                                <Grid.ColumnDefinitions>
                                                    <ColumnDefinition Width=""*"" />
                                                    <ColumnDefinition Width=""Auto"" />
                                                </Grid.ColumnDefinitions>
                                                <Grid Margin=""{TemplateBinding Padding}"">
                                                    <Border Background=""Black"" Opacity=""0"" />
                                                    <Border Background=""{TemplateBinding Background}"" BorderBrush=""{TemplateBinding BorderBrush}"" 
							                      BorderThickness=""{TemplateBinding BorderThickness}"" CornerRadius=""{TemplateBinding CornerRadius}""/>
                                                    <ContentPresenter 
                                                    Margin=""{TemplateBinding BorderThickness}""
                                                    HorizontalAlignment=""{TemplateBinding HorizontalContentAlignment}"" 
                                                    VerticalAlignment=""{TemplateBinding VerticalContentAlignment}"" 
                                                    Content=""{TemplateBinding Content}"" />
                                                </Grid>
                                            </Grid>
                                        </ControlTemplate>
                                    </Setter.Value>
                                </Setter>
                            </Style>")));

            RowStyle = (Style)XamlReader.Load(XmlReader.Create(new StringReader(@"<Style xmlns=""http://schemas.microsoft.com/winfx/2006/xaml/presentation""
                                 xmlns:x=""http://schemas.microsoft.com/winfx/2006/xaml"" 
                                 xmlns:vsm=""" + "clr-namespace:System.Windows;assembly=WPFToolkit" + @""" " +
                                 @"xmlns:c1=""clr-namespace:C1.WPF;assembly=C1.WPF""
                                 xmlns:c1DataGrid=""clr-namespace:C1.WPF.DataGrid;assembly=C1.WPF.DataGrid"" 
                                 TargetType=""c1DataGrid:DataGridSelectableRowPresenter"">
        <Setter Property=""Template"">
            <Setter.Value>
                <ControlTemplate TargetType=""c1DataGrid:DataGridSelectableRowPresenter"" >
                    <Grid x:Name=""Root"" Margin=""{TemplateBinding CellsAreaMargin}"">
                        <vsm:VisualStateManager.VisualStateGroups>
                            <vsm:VisualStateGroup x:Name=""CommonStates"">
                                <vsm:VisualState x:Name=""Normal""/>
                                <vsm:VisualState x:Name=""Disabled""/>
                                <vsm:VisualState x:Name=""MouseOver"">
                                    <Storyboard>
                                        <ObjectAnimationUsingKeyFrames Storyboard.TargetName=""MouseOverElement"" Storyboard.TargetProperty=""(UIElement.Visibility)"" Duration=""0"">
                                            <DiscreteObjectKeyFrame KeyTime=""00:00:00"">
                                                <DiscreteObjectKeyFrame.Value>
                                                    <Visibility>Visible</Visibility>
                                                </DiscreteObjectKeyFrame.Value>
                                            </DiscreteObjectKeyFrame>
                                        </ObjectAnimationUsingKeyFrames>
                                    </Storyboard>
                                </vsm:VisualState>
                            </vsm:VisualStateGroup>
                            <vsm:VisualStateGroup x:Name=""SelectionStates"">
                                <vsm:VisualState x:Name=""Editing"">
                                    <Storyboard>
                                        <ObjectAnimationUsingKeyFrames Storyboard.TargetName=""SelectedElement"" Storyboard.TargetProperty=""(UIElement.Visibility)"" Duration=""0"">
                                            <DiscreteObjectKeyFrame KeyTime=""00:00:00"">
                                                <DiscreteObjectKeyFrame.Value>
                                                    <Visibility>Visible</Visibility>
                                                </DiscreteObjectKeyFrame.Value>
                                            </DiscreteObjectKeyFrame>
                                        </ObjectAnimationUsingKeyFrames>
                                    </Storyboard>
                                </vsm:VisualState>
                                <vsm:VisualState x:Name=""Selected"">
                                    <Storyboard>
                                        <ObjectAnimationUsingKeyFrames Storyboard.TargetName=""SelectedElement"" Storyboard.TargetProperty=""(UIElement.Visibility)"" Duration=""0"">
                                            <DiscreteObjectKeyFrame KeyTime=""00:00:00"">
                                                <DiscreteObjectKeyFrame.Value>
                                                    <Visibility>Visible</Visibility>
                                                </DiscreteObjectKeyFrame.Value>
                                            </DiscreteObjectKeyFrame>
                                        </ObjectAnimationUsingKeyFrames>
                                    </Storyboard>
                                </vsm:VisualState>
                                <vsm:VisualState x:Name=""Unselected""/>
                            </vsm:VisualStateGroup>
                            <vsm:VisualStateGroup x:Name=""ValidationStates"">
                                <vsm:VisualState x:Name=""Valid""/>
                                <vsm:VisualState x:Name=""InvalidUnfocused"">
                                    <Storyboard>
                                        <DoubleAnimationUsingKeyFrames Storyboard.TargetName=""Validation"" Storyboard.TargetProperty=""(UIElement.Opacity)"" Duration=""0"">
                                            <DiscreteDoubleKeyFrame KeyTime=""00:00:00"" Value=""1""/>
                                        </DoubleAnimationUsingKeyFrames>
                                    </Storyboard>
                                </vsm:VisualState>
                                <vsm:VisualState x:Name=""InvalidFocused"" >
                                </vsm:VisualState>
                            </vsm:VisualStateGroup>
                        </vsm:VisualStateManager.VisualStateGroups>
                        <Grid.RowDefinitions>
                            <RowDefinition Height=""*"" />
                            <RowDefinition Height=""Auto"" />
                        </Grid.RowDefinitions>
						<c1:C1BrushBuilder x:Name=""SelectableBackgroundBrush"" Input=""{TemplateBinding BackgroundBrush}"" DesignBrush=""#FFD6E2ED"" DesignColor=""#FFC5D2DE""/>
                        <c1:C1BrushBuilder x:Name=""SelectedBrush"" Input=""{TemplateBinding SelectedBrush}"" DesignBrush=""#CC8DD6E8"" DesignColor=""#FF83CCDE""/>
                        <c1:C1BrushBuilder x:Name=""MouseOverBorderBrush"" Input=""{TemplateBinding MouseOverBrush}"" DesignBrush=""#B2A1D4E1"" DesignColor=""#FF83CCDE""/>
                        <Border x:Name=""Background"" Background=""{Binding ElementName=SelectableBackgroundBrush, Path=Output}"" BorderBrush=""{TemplateBinding BorderBrush}"" BorderThickness=""{TemplateBinding BorderThickness}"" />
                        <Border x:Name=""UserBackground"" Background=""{TemplateBinding Background}"" />
                        <Border x:Name=""Validation"" Background=""#2BFF0000"" Opacity=""0"" />
                        <Border x:Name=""SelectedElement"" Background=""{TemplateBinding SelectedBrush}"" BorderThickness=""{TemplateBinding BorderThickness}"" Visibility=""Collapsed"" />
                        <Border x:Name=""MouseOverElement"" Background=""{Binding ElementName=MouseOverBorderBrush, Path=Output}"" BorderThickness=""{TemplateBinding BorderThickness}"" Visibility=""Collapsed"" />
                        <c1DataGrid:DataGridCellsPanel x:Name=""Cells""/>
                        <Rectangle x:Name=""HorizontalGridLine"" Grid.Row=""1"" Height=""1"" Fill=""{TemplateBinding HorizontalGridLineBrush}"" Visibility=""{TemplateBinding HorizontalGridLineVisibility}""/>
                    </Grid>
                </ControlTemplate>
            </Setter.Value>
        </Setter>
    </Style>")));
        }

        #endregion
    }

}
